use http::StatusCode;
use serde::Serialize;

#[derive(Serialize)]
pub struct Reply<T>
where
    T: Serialize,
{
    #[serde(skip_serializing)]
    pub status: StatusCode,
    pub code: i32,
    pub err: bool,
    pub msg: String,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub data: Option<T>,
}

pub enum Status<T>
where
    T: Serialize,
{
    OK(Option<T>),
    Err(StatusCode, i32, String),
}

impl<T> Status<T>
where
    T: Serialize,
{
    pub fn to_reply(self) -> Reply<T> {
        let mut resp = Reply {
            status: StatusCode::OK,
            code: 0,
            err: false,
            msg: String::from("OK"),
            data: None,
        };

        match self {
            Status::OK(data) => {
                resp.data = data;
            }
            Status::Err(status, code, msg) => {
                resp.status = status;
                resp.code = code;
                resp.err = true;
                resp.msg = msg;
            }
        }

        resp
    }
}
